@@ -1,7 +1,9 @@
 // allowed.h: externs for allowed.c
 
-void InitAllowedFunc(SEXP Allowed, int nAllowedFuncArgs, SEXP Env,
-                     const char *sPredNames[], int nPreds);
+void InitAllowedFunc(
+        SEXP Allowed, // can be NULL
+        int nAllowedFuncArgs, SEXP Env,
+        const char *sPredNames[], int nPreds);
 
 void FreeAllowedFunc(void);
 
